#-*-coding:utf-8-*-

"""contient des fonctions de conversion et des
évaluations ou mesures d'usage général.
"""


import numpy as NY
try:
    from samplerate import sr
except ImportError:
    sr = 44100

__date__ = "2003.06.29, 2003.09.08, 2008.12.06"
__author__ = "Copyright (c) René Bastian 2002-2008 <rbastian@free.fr>"



cDiapason = 440.0

cC0 = ((cDiapason * (2**(1.0/12))**3) * 2**-6)
cDemiTon = 2**(1.0/12)
logcDemiTon = NY.log(cDemiTon)
logcC0 = NY.log(cC0)

gamme = {0 : "c", 2: "d", 4:"e", 5:"f", 7:"g", 9:"a", 11:"b"}
Gamme = {0 : "C", 2: "D", 4:"E", 5:"F", 7:"G", 9:"A", 11:"B"}

def freq2prony(freq):
    """convertit 'freq' de Hz en Prony"""
    return 12 * NY.log(freq / cDiapason)/NY.log(2.) + 69.

def prony2freq(prony):
    """convertit 'prony' de Prony en Hz"""
    return cC0 * cDemiTon**prony

def prony2periode(prony):
    """retourne la période"""
    return 1. / prony2freq(prony)

def prony2samples(prony):
    """retourne le nombre d'échantillons
    correspondant à prony. c'est utile ? oui !"""
    v = NY.array(sr / prony2freq(prony))
    
    return v.astype(NY.int32)

def freq2samples(freq):
    """retourne le nombre d'échantillons
    correspondant à prony. c'est utile ? oui !"""
    return int(sr / freq)

def freq2periode(freq):
    """convertit la frequence en période"""
    return 1. / freq

def samples2freq(n):
    """convertit 'n' samples en freq"""
    return float(sr) / n

def degre2radian(angle):
    """transforme degrés en radians"""
    return NY.pi * angle / 180.0

def radian2degre(angle):
    """transforme radians en degrés"""
    return angle * 180.0 / NY.pi

def sommeabs(a):
    """
    retourne la moyenne des valeurs absolues
    'a' : un vecteur
    """
    return sum(abs(a)) / len(a)

def extremum(x):
    """retourne la valeur extrême en valeur absolue."""
    return max(max(x), abs(min(x)))

def extremumstereo(x1, x2):
    """retourne l'extremum d'un vecteur stereo."""
    return max(extremum(x1), extremum(x2))



