#-*-coding:utf-8-*-

"""
module de la classe 'Enregistreur'
"""


import os
import numpy as np
import utiles 
import ajuster as _a

__author__ = """Copyright (c) 2005 René Bastian (Wissembourg, FRANCE)
et Alexandre Fayolle pour la méthode fusion"""
__date__ = "2005.01.07, 2005.10.24, 2006.01.26, 2006.02.04, 2006.11.06"

__all__ = ["Multi", "Enregistreur"]
        
class Enregistreur:
    """
    - classe pour enregistrer des signaux dans un fichier au format Float64
    donc les opérations au niveau du disque (tandis que 'partition'
    s'occupe du concept musical).
    """
    def __init__(self, nom, dureeTotale = 1.0, nouveau = 0):
        """
        Si le fichier 'nom' existe, ouvre le fichier
        pour lecture et écriture           
        Sinon, crée un fichier de zéros de durée minimum
        égale à 'dureeTotale'.
        @todo: il faudrait inverser dureeTotale et nouveau
        'nom' : nom du fichier
        'dureeTotale' : durée en secondes        
        """
        self.nom = nom
        if not os.path.exists(nom) or nouveau:
            self.dimension = long(dureeTotale*utiles.sr) + 1
            z = np.zeros(self.dimension, np.float64)
            z.tofile(nom)
        self.f = open(nom,"r+")
        self.dimension = os.fstat(self.f.fileno()).st_size/8
        self.maximum = None
                
    def close(self):
        """
        terminaison
        """
        self.f.close()
        
    def flush(self):
        """
        forcer l'écriture du fichier
        """
        self.f.flush()
        
    def afficher(self, debut, duree, standard = True):
        """
        Affiche un extrait du fichier
        'debut' : debut de la lecture en sec        
        'duree' : duree de la lecture en sec        
        """
        n = long(debut*utiles.sr)
        if n > self.dimension:
            print "debut  >  duree du fichier"
            return        
        d = long(duree*utiles.sr)
        if n+d > self.dimension:
            print "fin > duree du fichier"
            return
        self.f.seek(n*8)
        w = np.fromfile(self.f, np.float64, (d,))
        if standard :
            print w
        else :
            return w


    def dimensionTotale(self):
        """
        détermine la dimension du fichier
        double emploi avec '_dim...' ?
        """
        return os.fstat(self.f.fileno()).st_size/8

    def calcul_maximum(self):
        """détecte la valeur absolue maximale du vecteur"""
        d = os.stat(self.nom).st_size/8
        w = np.fromfile(self.f, np.float64, (d,))
        self.maximum = max(max(w), abs(min(w)))
        return self.maximum
                
    def ajouter(self, signal, pt):
        """
        Ajouter (additionner) un signal dans le fichier au
        point temporel donné.
        Cette fonction tient compte de ce que la fonction 'seek'
        prolonge automatiquement le fichier.
        'signal' : vecteur de nombres, numarray - Float64
        'pt' : point temporel en secondes, type float
        """
        if pt < 0.0 :
            print "Enregistreur:ajouter: Temps négatif !", pt
            return
        n = long(pt*utiles.sr)
        noctets = n*8
        d = len(signal)
        
        if self.dimension < (n+d):
            self._prolonger(n + d + 1)
            self.dimension = os.fstat(self.f.fileno()).st_size/8
        self.f.seek(noctets)
        try: 
            w = np.fromfile(self.f, np.float64, (d,))
        except ValueError:
            print "Enregistreur:ajouter:ValueError"
            print "noctets %20i dimension %20i signal %10i" % \
                  (noctets, self.dimension, d)
            return
        try :
            w = w + signal
        except StandardError:
            print "Enregistreur:ajouter", len(w), len(signal)
            return
        self.f.seek(noctets)
        w.tofile(self.f)
        self._dimension()

    def _dimension(self):
        """
        détermine la dimension du fichier
        """
        self.dimension = os.fstat(self.f.fileno()).st_size/8

    def _prolonger(self, necessaire):
        """
        prolonge le fichier pour atteindre 'necessaire' samples
        'necessaire' : nombre d'unités de type Float64 requises        
        """
        z = np.zeros(utiles.sr, np.float64)
        self.f.seek(necessaire*8)
        z.tofile(self.f)
        self._dimension()
        return

class Multi:
    """
    installe un enregistreur multipistes
    en env 600 sec compile une séquence de 1200 seq    
    """
    def __init__(self, nomsouche, npistes, dureeInitiale = 1.0,
                 nouveau=True):
        """
        'nomsouche' : nom générique pour les fichiers
        'npistes' : nombre de pistes
        'nouveau' :
            si True, l'existant est effacé
            si False, les signaux sont ajoutés à l'existant
        """
        if nouveau :
            nouveau = 1
        else:
            nouveau = 0
        self.pistes = []
        for i in xrange(npistes):
            fnom = "".join([nomsouche, ".", `i`, "f64"])
            f = Enregistreur(fnom, dureeInitiale, nouveau)
            self.pistes.append(f)        
        self.ext = 0
        self.npistes = npistes
        self.nomsouche = nomsouche
        
    def ajouter(self, pt, ncanal, signal):
        """
        ajoute le 'signal' au point temporel 'pt' dans le
        canal 'ncanal'.
        """        
        self.pistes[ncanal].ajouter(signal, pt)
        
    def ajouterfichier(self, pt, ncanal, fsignal):
        """
        ajoute le fichier audio 'fsignal' (format Float64)
        à la piste 'ncanal'
        au point temporel 'pt'
        """        
        signal = np.fromfile(fsignal, np.float64)
        self.pistes[ncanal].ajouter(signal, pt)
        
    def flush(self):
        """
        fait que tout soit enregistré
        """
        for f in self.pistes:
            f.flush()
            
    def close(self):
        """
        clôt toutes les pistes ouvertes
        """
        for f in self.pistes:
            f.close()
            
    def fermerpistes(self): 
        """
        comme 'close'
        """
        for f in self.pistes:
            f.close()

    def fusion(self, plafond=32767.0):
        """
        lit les fichiers 'self.nomsouche'+"?f64" et en fait un
        fichier 'self.nomsouche'+"raw".
        'plafond' : doit permettre de fixer la valeur maximale
        à un valeur inférieure à 32767.0 pour que les pièces moins
        dynamiques ne soient pas portées d'office à un niveau
        absolu trop élevé.
        Un autre moyen serait d'enregistrer une impulsion unique
        à 32767.0 et de la faire disparaître lors du gravage final.
        """
        F = []
        for i in range(self.npistes):
            F.append(self.nomsouche+"." + `i` + "f64")
        
        self.ext = 0.0
        for i in range(self.npistes):
            a = np.fromfile(F[i], np.float64)                        
            self.ext = max(a.max(), abs(a.min()), self.ext)            
        
        X = []
        for i in range(self.npistes):
            a = np.fromfile(F[i], np.float64)                        
            X.append(a)
        
        X = _a.prolongerZ(*X)
        X = np.array(X)
        try:
            X *= (plafond / self.ext)
        except ZeroDivisionError:
            print "fusion:ZeroDivisionError:"
            print max(a), min(a), self.ext
        X = X.astype(np.int16)
        X = np.transpose(X)        
        X.tofile(self.nomsouche + ".raw")


