#-*-coding:utf-8-*-

"""module d'une classe Audio pour
-- écouter un couple de deux vecteurs (play)
-- écrire un couple de deux vecteurs sur le disque dur en int16 (fraw)
-- écrire deux fichiers en float64 (f64)
-- écrire deux fichiers en float64 (txt) lisible et affichable par gnuplot
-- écouter et écrire (int16, float64) à la demande,
-- jouer une liste de fichiers 'raw'.

Fonctionne seulement s/Linux."""

__date__ = "2006.01.21, 2006.01.29, 2006.09.14"
__author__ = "Copyright (c) 2006 René Bastian (Wissembourg, FRANCE)"

__all__ = ["Audio"]

import ossaudiodev as OSS
import numpy as NY
import os
try:
    from utiles import sr
except StandardError:
    sr = 44100
import ajuster

import sys

class Audio:
    """classe pour envoyer - après conversion -
    des sons 64 bits à la carte son.
    -- écouter un couple de deux vecteurs (play)
    -- écrire un couple de deux vecteurs sur le disque dur en int16 (fraw)
    -- écrire deux fichiers en float64 (f64)
    -- écrire deux fichiers en float64 (txt) lisible et affichable par gnuplot
    -- écouter et écrire (int16, float64) à la demande.
    Fonctionne seulement s/Linux."""
    
    def __init__(self, canaux = 2, format = OSS.AFMT_S16_LE):
        """instancie une sortie stéréo.
        Inutile de faire un try, puisque ça n'ira pas ..."""
        if canaux != 2 :
            print u"Audio : version stéréo uniquement"
            canaux = 2
        try:
            self.f = OSS.open("w")
        except IOError:
            print "OSS: ou bien je n'arrive pas à ouvrir l'interface audio"
            print "OSS: ou bien elle est déjà ouverte ... "
            self.ossflag = False            
            sys.exit()    
        self.ossflag = True
        self.f.setfmt(format)
        self.f.channels(canaux)
        self.f.speed(sr)
        self.len = 0
        
    def close(self):
        """ferme la sortie stéréo."""
        try:
            self.f.close()
        except IOError:
            print "live.close: ne fonctionne pas"
    
    def play(self, wg, wd, egalise = True, test = False, amorti=1.0):
        """
        convertit les vecteurs 64 bits et les envoie sur la sortie.
        'wg, wd' : canal gauche et droit 
        'egalise = True' 
        'test = False' : contrôle si les vecteurs ne comportent pas des 'nan' 
        et autres facéties 
        'amorti=1.0' : nécessaire parce que certains sons sont fortement 
        distordus si l'amplification va au maximum possible.
        """
        
        if len(wg) != len(wd):
            wg, wd = ajuster.prolongerZ(wg, wd)
        self.len = float(len(wg)) / sr
        c = (wg, wd)
        if test:
            for w in c:
                for x in w:                
                    
                    if x in [NY.nan, NY.inf]:                             
                        print "Voilà !", x
        c = NY.transpose(c)
        c = NY.ravel(c)
        if egalise :
            m = NY.maximum(c.max(), NY.abs(c.min()))
            if m in [NY.nan, NY.inf]:
                print "play:'if egalise': on divise par 0"
                return None
            c = (c * 32767.0 * amorti) / m
        else:
            c = c * 32767.0 * amorti
        try:
            c = c.astype(NY.int16)
        except StandardError:
            print "live:play:astype"
        if self.ossflag :
            self.f.write(c)
        else :
            print """Ce serait étonnant que ça marche :-) :
            il faut d'abord fermer l'application qui occupe la sortie audio."""
            print "Audio.play.ossflag", self.ossflag
            c.tofile("x.raw")
            os.system("sox -c2 -r44100 -sw -fs x.raw x.wav")
            os.system("aplay x.wav")

    def _conforme(self, wg, wd):
        """
        tricote le fichier stéréo à partir de deux
        canaux unidimensionnels.
        """
        c = (wg, wd)
        c = NY.transpose(c)
        c = NY.ravel(c)
        m = NY.maximum(c.max(), NY.abs(c.min()))
        if m == 0.0 :
            print "Audio._conforme : maximum == 0.0"
        else :
            c = c * 32767.0 / m
        try:
            c = NY.array(c, NY.int16)
        except StandardError:
            print "live:_conforme:NY.array(c, NY.int16) quoi ??"
        return c

    def fraw(self, wg, wd, nom = "x"):
        """écrit wg et wd dans un fichier stéréo en format 'raw'.""" 
        if len(wg) != len(wd):
            wg, wd = ajuster.prolongerZ(wg, wd)
        c = self._conforme(wg, wd)
        c.tofile(nom+".raw")
        comm = " ".join(["sox -c2 -r44100 -sw", nom+".raw", nom+".wav"])
        try:
            os.system(comm)
        except StandardError:
            print "sox : la conversion en fichier wav n'a pas pu être faite." 
        return

    def fwav(self, wg, wd, nom = "x"):
        """écrit wg et wd dans un fichier stéréo
        converti au format 'wav'.""" 
        if len(wg) != len(wd):
            wg, wd = ajuster.prolongerZ(wg, wd)
        c = self._conforme(wg, wd)
        c.tofile(nom+".raw")
        comm = " ".join(["sox -c2 -r44100 -sw", nom+".raw", nom+".wav"])
        try:
            os.system(comm)
            os.system("rm "+nom+".raw")
        except StandardError:
            print "sox : la conversion en fichier wav n'a pas pu être faite." 
        return

    def rec(self, wg, wd, nom64, nomwav):
        """
        enregistre en format 'w' et 'f64'
        'nom64', 'nomwav' doivent être des noms-souches sans les suffixes
        """
        self.fwav(wg, wd, nomwav)
        wg.tofile(nom64 + ".0f64")
        wd.tofile(nom64 + ".1f64")
        


